TRDsum := proc(s1::table, s2::table)
    local homog_parts_of_sum := proc(d::integer)
          return (TRDhomogeneous_part(s1, d) + TRDhomogeneous_part(s2, d));
    end proc;
    TRDmake_power_series(TRDtruncate_polynomial(s1[POLY],min(s1[DEG], s2[DEG])) +
                         TRDtruncate_polynomial(s2[POLY],min(s1[DEG], s2[DEG])),
                         min(s1[DEG], s2[DEG]),
                         homog_parts_of_sum);
end proc;

TRDsub := proc(s1::table, s2::table)
    local homog_parts_of_sub := proc(d::integer)
          return (TRDhomogeneous_part(s1, d) - TRDhomogeneous_part(s2, d));
    end proc;
    TRDmake_power_series(TRDtruncate_polynomial(s1[POLY],min(s1[DEG], s2[DEG])) -
                         TRDtruncate_polynomial(s2[POLY],min(s1[DEG], s2[DEG])),
                         min(s1[DEG], s2[DEG]),
                         homog_parts_of_sub);
end proc;

TRDhomogeneous_part := proc(ps::table, d::integer)
local i, h;
    if (d > ps[DEG]) then
        for i from (ps[DEG] + 1) to d do
            h := (ps[GEN])(i);
            ps[POLY] := ps[POLY] + h
        end do;
        ps[DEG] := d;
        return (h);
    end if;
    return TRDhomogeneous_part_of_polynomial(ps[POLY], d);
end proc;

TRDpolynomial_part := proc(ps::table, d::integer)
local i, h;
    if (d > ps[DEG]) then
        for i from (ps[DEG] + 1) to d do
            h := (ps[GEN])(i);
            ps[POLY] := ps[POLY] + h
        end do;
        ps[DEG] := d;
        return (ps[POLY]);
     elif (d = ps[DEG]) then
        return (ps[POLY]);
     else
        return TRDtruncate_polynomial(ps[POLY], d);
     end if;
end proc;

TRDproduct := proc(s1::table, s2::table)
    local homog_parts_of_prod := proc(d::integer)
          local p, i;
              p := 0;
              for i from 0 to d do
                  p := p + (TRDhomogeneous_part(s1, i) * TRDhomogeneous_part(s2, d-i));
              end do;
          return (p);
    end proc;
    TRDmake_power_series(TRDtruncate_polynomial(s1[POLY] * s2[POLY],min(s1[DEG], s2[DEG])),
                         min(s1[DEG], s2[DEG]),
                         homog_parts_of_prod);
end proc;

TRDexact_quotient := proc(f::table, h::table)
    local homog_parts_of_quotient := proc(d::integer) option remember;
          local f_0, f_1, h_0, h_1, g_0, g_1, s, p, g_d, e, i;
          h_0 := TRDhomogeneous_part(h, 0);
          if (h_0 = 0) then error "Invalid division"; end if;
          ######################
          ## First two values ##
          ######################
          f_0 := TRDhomogeneous_part(f, 0);
          g_0 := normal(f_0 / h_0);
          if (d = 0) then return (g_0); end if;
          h_1 := TRDhomogeneous_part(h, 1);
          f_1 := TRDhomogeneous_part(f, 1);
          g_1 := normal((f_1 - g_0 * h_1)  / h_0);
          g_1 := TRDhomogeneous_part_of_polynomial(g_1, 1);
          if (d = 1) then return (g_1); end if;
          ##################
          ## Other values ##
          ##################
          s := TRDhomogeneous_part(f,d);
          for i from 1 to d do
              p := TRDhomogeneous_part(h, i) * homog_parts_of_quotient(d-i);
              s := s - p ;
          end do;
          g_d := normal(s / h_0);
          g_d := TRDhomogeneous_part_of_polynomial(g_d, d);
          return (g_d);
     end proc;
     TRDmake_power_series(homog_parts_of_quotient(0),
                          0, homog_parts_of_quotient);
end proc;
