read "powerseries.mpl":
PS := PowerSeries([X,Y]):
with(PS):
UPoPS := UnivariatePolynomialOverPowerSeries([X,y],Z):
with(UPoPS):
u1 := UPoPS:-One:
u0 := UPoPS:-Zero:
s1 := PS:-One:
s0 := PS:-Zero:

test_1 := proc()
    local s2, s3, hp, s4, s5, a, u2, p, alpha;
    s2 := PS:-PolynomialToPowerSeries(1 - X);
    s3 := PS:-ExactQuotient(s1, s2);
    hp := PS:-HomegeneousPart(s3,2);
    if (hp <> X^2) then
         error "Wrong result";
    end if;
    s4 := PS:-PolynomialToPowerSeries(1 - Y);
    s5 := PS:-Inverse(s4);
    hp := PS:-HomegeneousPart(s5,3):
    if (hp <> Y^3) then
         error "Wrong result";
    end if;
    a := Array([s3, s5]);
    u2 := UPoPS:-FromArrayOfPowerSeries(a);
    UPoPS:-ComputedTerms(u2);
    (p, alpha) := UPoPS:-WeierstrassPreparation(u2, 2);
    if (PS:-ComputedTerms(p) <> 1) then
         error "Wrong result";
    end if;
    PS:-ComputedTerms(alpha);
    print ("Passed!");
end proc:

test_2 := proc()
    local s4, s5, s6, a, u2, p, alpha, residual;
    s4 := PS:-PolynomialToPowerSeries(1 - Y);
    s5 := PS:-Inverse(s4);
    s6 := PS:-PolynomialToPowerSeries(X);
    a := Array([s6, s5]);
    u2 := UPoPS:-FromArrayOfPowerSeries(a);
    if (PolynomialPart(u2, 2) <> X+(Y^2+Y+1)*Z) then
         error "Wrong result";
    end if;
    UPoPS:-ComputedTerms(u2);
    (p, alpha) := UPoPS:-WeierstrassPreparation(u2, 2);
    PS:-ComputedTerms(p);
    PS:-ComputedTerms(alpha);
    residual := UPoPS:-Subtract(u2,UPoPS:-Multiply(p,alpha));
    if (UPoPS:-PolynomialPart(residual, 2) <> 0) then
         error "Wrong result";
    end if;
    print ("Passed!");
end proc:

## Example 1: Correct
test_3  := proc()
    local u7, p, alpha, residual;
    u7 := UPoPS:-FromListOfPolynomials([X, 1]);
    UPoPS:-PolynomialPart(u7,2);
    (p, alpha) := UPoPS:-WeierstrassPreparation(u7, 2);
    UPoPS:-PolynomialPart(p,2);
    UPoPS:-PolynomialPart(alpha,2);
    residual := UPoPS:-Subtract(u7,UPoPS:-Multiply(p,alpha));
    if (UPoPS:-PolynomialPart(residual, 2) <> 0) then
         error "Wrong result";
    end if;
    print ("Passed!");
end proc:

## Example 2: Correct
test_4  := proc()
    local u8, p, alpha, residual;
    u8 := UPoPS:-FromListOfPolynomials([1, X]);
    UPoPS:-PolynomialPart(u8,2);
    (p, alpha) := UPoPS:-WeierstrassPreparation(u8, 2);
    UPoPS:-PolynomialPart(p,2);
    UPoPS:-PolynomialPart(alpha,2);
    residual := UPoPS:-Subtract(u8,UPoPS:-Multiply(p,alpha));
    if (UPoPS:-PolynomialPart(residual, 2) <> 0) then
         error "Wrong result";
    end if;
    print ("Passed!");
end proc:

## Example 3: Correct
test_5  := proc()
    local u9, p, alpha, residual;
    u9 := UPoPS:-FromListOfPolynomials([Y, 1, X]);
    UPoPS:-PolynomialPart(u9,2);
    (p, alpha) := UPoPS:-WeierstrassPreparation(u9, 2);
    UPoPS:-PolynomialPart(p,2);
    UPoPS:-PolynomialPart(alpha,2);
    (p, alpha) := UPoPS:-WeierstrassPreparation(u9, 4);
    UPoPS:-PolynomialPart(p,4);
    UPoPS:-PolynomialPart(alpha,4);
    residual := UPoPS:-Subtract(u9,UPoPS:-Multiply(p,alpha));
    UPoPS:-PolynomialPart(residual, 4);
    if (UPoPS:-PolynomialPart(residual, 4) <> 0) then
        error "Wrong result";
    end if;
    print ("Passed!");
end proc:

## Example 4: Correct
test_6  := proc()
    local u10, p, alpha, residual;
    u10 := UPoPS:-FromListOfPolynomials([Y, 1, X+1]);
    UPoPS:-PolynomialPart(u10,2);
    (p, alpha) := UPoPS:-WeierstrassPreparation(u10, 2);
    UPoPS:-PolynomialPart(p,2);
    UPoPS:-PolynomialPart(alpha,2);
    residual := UPoPS:-Subtract(u10,UPoPS:-Multiply(p,alpha));
    UPoPS:-PolynomialPart(residual, 2);
    if (UPoPS:-PolynomialPart(residual, 2) <> 0) then
        error "Wrong result";
    end if;
    (p, alpha) := UPoPS:-WeierstrassPreparation(u10, 4);
    UPoPS:-PolynomialPart(p,4);
    UPoPS:-PolynomialPart(alpha,4);
    residual := UPoPS:-Subtract(u10,UPoPS:-Multiply(p,alpha));
    UPoPS:-PolynomialPart(residual,4);
    if (UPoPS:-PolynomialPart(residual, 4) <> 0) then
        error "Wrong result";
    end if;
    (p, alpha) := UPoPS:-WeierstrassPreparation(u10, 30);
    UPoPS:-PolynomialPart(p,30);
    UPoPS:-PolynomialPart(alpha,30);
    residual := UPoPS:-Subtract(u10,UPoPS:-Multiply(p,alpha));
    UPoPS:-PolynomialPart(residual,30);
    if (UPoPS:-PolynomialPart(residual, 30) <> 0) then
        error "Wrong result";
    end if;

    print ("Passed!");
end proc:
